"""
Python-equivalent of ``avif_example_decode_file.c``.
"""

import sys

from _avif import ffi, lib
from avif import Decoder


def main():
    filename = sys.argv[1]

    rgb = ffi.new("avifRGBImage*")
    decoder = Decoder()

    image = decoder.parse_path(filename)
    print(f"Parsed AVIF: {image.width}x{image.height} ({image.depth}bpc)")

    for image in decoder.next_images():
        lib.avifRGBImageSetDefaults(rgb, image)
        lib.avifRGBImageAllocatePixels(rgb)
        lib.avifImageYUVToRGB(image, rgb)

        pixels = rgb.pixels
        if rgb.depth > 8:
            pixels = ffi.cast("uint16_t *", pixels)

        first_pixel = tuple(pixels[0:4])
        print(f" * First pixel: RGBA{first_pixel}")


if __name__ == "__main__":
    main()



import os

import attr

from _avif import ffi, lib


class AVIFError(Exception):
    pass


def _succeed(avif_result):
    """
    Raise an exception if a ``libavif`` library function failed.
    """

    if avif_result != lib.AVIF_RESULT_OK:
        c_string = lib.avifResultToString(avif_result)
        raise AVIFError(ffi.string(c_string).decode("utf-8"))


def _avifDecoder():
    """
    Create an FFI-managed avifDecoder.
    """

    return ffi.gc(lib.avifDecoderCreate(), lib.avifDecoderDestroy)


@attr.s
class Decoder:
    """
    An AVIF decoder.
    """

    _decoder = attr.ib(factory=_avifDecoder)

    def parse_path(self, path):
        _succeed(lib.avifDecoderSetIOFile(self._decoder, os.fsencode(path)))
        _succeed(lib.avifDecoderParse(self._decoder))
        return self._decoder.image

    def next_images(self):
        while lib.avifDecoderNextImage(self._decoder) == lib.AVIF_RESULT_OK:
            yield self._decoder.image
